class DemanderStatus < ActiveRecord::Base
  has_one :demander
  
#  validates :token
  validates :receive_hour1, :numericality => {:allow_blank => true}
  validates :receive_hour2, :numericality => {:allow_blank => true}
  
  def supply_default
  end
  
  def overwrite
  end
  
  def own? ad
    return false unless ad
    ad.is_a?(Admin)
  end
  
  def visible? ad
    return false unless ad
    ad.is_a?(Admin)
  end
  
  def status
    return 0 if self.requested_at.blank?
    return 1 if self.requested_at and self.rejected_at
    return 3 if self.requested_at and self.approved_at
    return 2 if self.requested_at and self.rejected_at.blank? and self.approved_at.blank?
    nil
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(DemanderStatus.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:order => 'demanders.name'})
    DemanderStatus.find(:all, opt)
  end
  
  def self.requested_list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(DemanderStatus.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'demander_statuses.requested_at is not null and demander_statuses.approved_at is null and demander_statuses.rejected_at is null', :order => 'demanders.name'})
    DemanderStatus.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:demander => {}} }
  end
  
  def self.list_json_opt
    {:include => {:demander => {}} }
  end
  
  def self.show pid, ad
    opt = {}
    opt.merge!(DemanderStatus.show_opt)
    res = DemanderStatus.find(pid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(ad)
    res
  end
  
  def self.edit pid, ad
    opt = {}
    opt.merge!(DemanderStatus.show_opt)
    res = DemanderStatus.find(pid, opt)
    raise ActiveRecord::Forbidden unless res.own?(ad)
    res
  end
  
  def self.show_opt
    {:include => {:demander => {}} }
  end
  
  def self.show_json_opt
    {:include => {:demander => {}} }
  end
  
  def request
    return false if self.status == 2 or self.status == 3
    self.requested_at = Time.now
    self.rejected_at = nil
    self.approved_at = nil
    self.save
  end
  
  def reject
    return false if self.status == 0 or self.status == 1
    r = false
    DemanderStatus.transaction do
      self.rejected_at = Time.now
      self.approved_at = nil
      r = self.demander.demand_user.delete_token
      raise ActiveRecord::Rollback unless r
      r = self.save
    end
    r
  end
  
  def approve h1, h2
    return false if self.status == 0 or self.status == 3
    r = false
    DemanderStatus.transaction do
      self.rejected_at = nil
      self.approved_at = Time.now
      self.receive_hour1 = h1
      self.receive_hour2 = h2
      r = self.demander.demand_user.create_token
      raise ActiveRecord::Rollback unless r
      r = self.save
    end
    r
  end
  
  def stop
    r = false
    DemanderStatus.transaction do
      self.requested_at = nil
      self.rejected_at = nil
      self.approved_at = nil
      self.receive_hour1 = nil
      self.receive_hour2 = nil
      r = self.demander.demand_user.delete_token
      raise ActiveRecord::Rollback unless r
      r = self.save
    end
    r
  end
  
end
